@@ -537,7 +537,7 @@ process_file (FTS *fts, FTSENT *ent)
       duinfo_set (&dui,
 		  (apparent_size
 		   ? sb->st_size
-		   : ST_NBLOCKS (*sb) * ST_NBLOCKSIZE),
+		   : (uintmax_t) ST_NBLOCKS (*sb) * ST_NBLOCKSIZE),
 		  (time_type == time_mtime ? get_stat_mtime (sb)
 		   : time_type == time_atime ? get_stat_atime (sb)
 		   : get_stat_ctime (sb)));
